/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.engineReleases;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.device.api.engineReleases.service.CdEngineReleasesApiService;
import net.northking.cctp.device.dto.engineReleases.CdEngineReleasesAddDto;
import net.northking.cctp.device.dto.engineReleases.CdEngineReleasesDetailDto;
import net.northking.cctp.device.dto.engineReleases.CdEngineReleasesUpdateDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 执行引擎版本 => Restful 类型 HTTP/HTTPS 接口
 *
 * @author maven-cctp-plugin
 * @since 1.0
 */
@Api(tags = "执行引擎版本")
@RequestMapping("/v1/CdEngineReleasess")
@RestController
public class CdEngineReleasesRestfulCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(CdEngineReleasesRestfulCtrl.class);

  @Autowired
  private CdEngineReleasesApiService apiService;

  /**
   * Restful接口-post： 新增执行引擎版本
   *
   * @param inputDto         接口数据传输对象
   * @return 数据库主键
   */
  @ApiOperation(value = "新增：执行引擎版本")
  @PostMapping(
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<String> add(
      @RequestBody @Validated CdEngineReleasesAddDto inputDto)
  {
    ResultWrapper<String> wrapper = new ResultWrapper<>();
    String uuid = apiService.add(inputDto);
    return wrapper.success(uuid);
  }

  /**
   * Restful接口-put： 更新执行引擎版本
   *
   * @param inputDto         接口数据传输对象
   * @return 修改的记录数量
   */
  @ApiOperation(value = "更新：执行引擎版本")
  @PutMapping(
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> update(
      @RequestBody @Validated CdEngineReleasesUpdateDto inputDto) throws IllegalAccessException
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.updateByPK(inputDto);
    return wrapper.success(count);
  }

  /**
   * Restful接口-get： 获取执行引擎版本
   *
   * @param uuid       数据库主键
   * @return 接口数据传输对象
   */
  @ApiOperation(value = "获取：执行引擎版本")
  @GetMapping(
      value = "/{uuid}",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<CdEngineReleasesDetailDto> findByPK(
      @PathVariable("uuid") String uuid)
  {
    ResultWrapper<CdEngineReleasesDetailDto> wrapper = new ResultWrapper<>();
    CdEngineReleasesDetailDto user = apiService.findByPK(uuid);
    return wrapper.success(user);
  }

  /**
   * Restful接口-delete： 删除执行引擎版本
   *
   * @param uuid       数据库主键
   * @return 删除的记录数量
   */
  @ApiOperation(value = "删除：执行引擎版本")
  @DeleteMapping(
      value = "/{uuid}",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Integer> deleteByPK(
      @PathVariable("uuid") String uuid)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    Integer count = apiService.deleteByPK(uuid);
    return wrapper.success(count);
  }
}
