/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.atu.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.db.entity.DwAtuScriptVersion;
import net.northking.cctp.dw.dto.atu.DwAtuScriptVersionDetailDto;
import net.northking.cctp.dw.dto.atu.DwAtuScriptVersionQueryDto;
import net.northking.cctp.dw.dto.atu.DwAtuScriptVersionUpdateDto;

import java.util.List;

/**
 * dw脚本版本同步表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface DwAtuScriptVersionApiService extends ExcelService
{
  /**
   * 查询dw脚本版本同步表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<DwAtuScriptVersionDetailDto> pagingQuery(QueryByPage<DwAtuScriptVersionQueryDto> queryByPage);

  /**
   * 新增dw脚本版本同步表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(DwAtuScriptVersion dto);

  /**
   * 更新dw脚本版本同步表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(DwAtuScriptVersionUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取dw脚本版本同步表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  DwAtuScriptVersionDetailDto findByPK(String uuid);

  /**
   * 删除dw脚本版本同步表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<DwAtuScriptVersion> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<DwAtuScriptVersionUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    Integer deleteByExamples(List<DwAtuScriptVersion> dwAtuScriptVersionDeletes);
}
