/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.db.entity.BugTemplateStatus;
import net.northking.cctp.dw.dto.BugTemplateStatusAddDto;
import net.northking.cctp.dw.dto.BugTemplateStatusDetailDto;
import net.northking.cctp.dw.dto.BugTemplateStatusQueryDto;
import net.northking.cctp.dw.dto.BugTemplateStatusUpdateDto;

import java.util.List;

/**
 * Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugTemplateStatusApiService extends ExcelService
{
  /**
   * 查询列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugTemplateStatusDetailDto> pagingQuery(QueryByPage<BugTemplateStatusQueryDto> queryByPage);

  /**
   * 新增
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugTemplateStatusAddDto dto);

  /**
   * 更新
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugTemplateStatusUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugTemplateStatusDetailDto findByPK(String uuid);

  /**
   * 删除
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugTemplateStatusAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugTemplateStatusUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  /**
   * 查询缺陷终态
   * @return
   */
  List<BugTemplateStatus> queryBugEndStatus();
}
